@@ -20,7 +20,7 @@
  * Author:      Larry Isaacs <larryi@apache.org>                           *
  * Author:      Ignacio J. Ortega <nacho@apache.org>                       *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 1.34 $                                          *
+ * Version:     $Revision: 1.55 $                                          *
  ***************************************************************************/
 
 // This define is needed to include wincrypt,h, needed to get client certificates
@@ -37,12 +37,13 @@
 #include "jk_service.h"
 #include "jk_worker.h"
 #include "jk_uri_worker_map.h"
+#include "jk_shm.h"
 
 #define VERSION_STRING "Jakarta/ISAPI/" JK_VERSTRING
-
+#define SHM_DEF_NAME   "JKISAPISHMEM"
 #define DEFAULT_WORKER_NAME ("ajp13")
 /*
- * We use special headers to pass values from the filter to the 
+ * We use special headers to pass values from the filter to the
  * extension. These values are:
  *
  * 1. The real URI before redirection took place
@@ -50,15 +51,25 @@
  * 3. The contents of the Translate header, if any
  *
  */
-#define URI_HEADER_NAME              ("TOMCATURI:")
-#define QUERY_HEADER_NAME            ("TOMCATQUERY:")
-#define WORKER_HEADER_NAME           ("TOMCATWORKER:")
-#define TOMCAT_TRANSLATE_HEADER_NAME ("TOMCATTRANSLATE:")
-#define CONTENT_LENGTH               ("CONTENT_LENGTH:")
+#define URI_HEADER_NAME_BASE              ("TOMCATURI")
+#define QUERY_HEADER_NAME_BASE            ("TOMCATQUERY")
+#define WORKER_HEADER_NAME_BASE           ("TOMCATWORKER")
+#define TOMCAT_TRANSLATE_HEADER_NAME_BASE ("TOMCATTRANSLATE")
+#define CONTENT_LENGTH                    ("CONTENT_LENGTH:")
+/* The template used to construct our unique headers
+ * from the base name and module instance
+ */
+#define HEADER_TEMPLATE      ("%s%p:")
+#define HTTP_HEADER_TEMPLATE ("HTTP_%s%p")
 
-#define HTTP_URI_HEADER_NAME         ("HTTP_TOMCATURI")
-#define HTTP_QUERY_HEADER_NAME       ("HTTP_TOMCATQUERY")
-#define HTTP_WORKER_HEADER_NAME      ("HTTP_TOMCATWORKER")
+static char URI_HEADER_NAME[_MAX_FNAME];
+static char QUERY_HEADER_NAME[_MAX_FNAME];
+static char WORKER_HEADER_NAME[_MAX_FNAME];
+static char TOMCAT_TRANSLATE_HEADER_NAME[_MAX_FNAME];
+
+static char HTTP_URI_HEADER_NAME[_MAX_FNAME];
+static char HTTP_QUERY_HEADER_NAME[_MAX_FNAME];
+static char HTTP_WORKER_HEADER_NAME[_MAX_FNAME];
 
 #define REGISTRY_LOCATION       ("Software\\Apache Software Foundation\\Jakarta Isapi Redirector\\1.0")
 #define EXTENSION_URI_TAG       ("extension_uri")
@@ -68,6 +79,10 @@
 #define URI_SELECT_UNPARSED_VERB    ("unparsed")
 #define URI_SELECT_ESCAPED_VERB     ("escaped")
 
+#define TRANSLATE_HEADER                              ("Translate:")
+#define TRANSLATE_HEADER_NAME                         ("Translate")
+#define TRANSLATE_HEADER_NAME_LC                      ("translate")
+
 #define BAD_REQUEST     -1
 #define BAD_PATH        -2
 #define MAX_SERVERNAME  128
@@ -78,13 +93,13 @@
                                 "Your browser (or proxy) sent a request that "                      \
                                 "this server could not understand.</DL></DD></BODY></HTML>"
 
-#define HTML_ERROR_403          "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"  \
-                                "<HTML><HEAD><TITLE>Access forbidden!!</TITLE></HEAD>"              \
-                                "<BODY><H1>Access forbidden!</H1><DL><DD>\n"                        \
-                                "You don't have permission to access the requested object."         \
-                                "It is either read-protected or not readable by the server.</DL></DD></BODY></HTML>"
+#define HTML_ERROR_404          "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"  \
+                                "<HTML><HEAD><TITLE>Object not found!</TITLE></HEAD>"               \
+                                "<BODY><H1>The requested URL was not found on this server"          \
+                                "</H1><DL><DD>\nIf you entered the URL manually please check your"  \
+                                "spelling and try again.</DL></DD></BODY></HTML>"
+
 
- 
 #define JK_TOLOWER(x)   ((char)tolower((BYTE)(x)))
 
 #define GET_SERVER_VARIABLE_VALUE(name, place)          \
@@ -94,8 +109,7 @@
     if (get_server_value(private_data->lpEcb,           \
                         (name),                         \
                         huge_buf,                       \
-                        huge_buf_sz,                    \
-                        "")) {                          \
+                        huge_buf_sz)) {                 \
         (place) = jk_pool_strdup(&private_data->p,      \
                                  huge_buf);             \
   } } while(0)
@@ -106,8 +120,7 @@
     if (get_server_value(private_data->lpEcb,               \
                         (name),                             \
                         huge_buf,                           \
-                        huge_buf_sz,                        \
-                        "")) {                              \
+                        huge_buf_sz)) {                     \
         (place) = atoi(huge_buf);                           \
         if (0 == (place)) {                                 \
             (place) = def;                                  \
@@ -123,6 +136,7 @@ static int is_mapread = JK_FALSE;
 static int iis5 = -1;
 
 static jk_uri_worker_map_t *uw_map = NULL;
+static jk_map_t *workers_map = NULL;
 static jk_logger_t *logger = NULL;
 static char *SERVER_NAME = "SERVER_NAME";
 static char *SERVER_SOFTWARE = "SERVER_SOFTWARE";
@@ -133,7 +147,7 @@ static char extension_uri[INTERNET_MAX_URL_LENGTH] =
 static char log_file[MAX_PATH * 2];
 static int log_level = JK_LOG_EMERG_LEVEL;
 static char worker_file[MAX_PATH * 2];
-static char worker_mount_file[MAX_PATH * 2];
+static char worker_mount_file[MAX_PATH * 2] = {0};
 
 #define URI_SELECT_OPT_PARSED       0
 #define URI_SELECT_OPT_UNPARSED     1
@@ -153,6 +167,11 @@ struct isapi_private_data_t
     LPEXTENSION_CONTROL_BLOCK lpEcb;
 };
 
+typedef struct isapi_log_data_t isapi_log_data_t;
+struct isapi_log_data_t {
+    char uri[INTERNET_MAX_URL_LENGTH];
+    char query[INTERNET_MAX_URL_LENGTH];
+};
 
 static int JK_METHOD start_response(jk_ws_service_t *s,
                                     int status,
@@ -181,7 +200,7 @@ static int get_registry_config_parameter(HKEY hkey,
 
 static int get_server_value(LPEXTENSION_CONTROL_BLOCK lpEcb,
                             char *name,
-                            char *buf, DWORD bufsz, char *def_val);
+                            char *buf, DWORD bufsz);
 
 static int base64_encode_cert_len(int len);
 
@@ -332,6 +351,61 @@ static BYTE *c2x(unsigned int what, BYTE *where)
     return where;
 }
 
+static char *status_reason(int status)
+{
+    static struct reasons {
+        int status;
+        char *reason;
+    } *r, reasons[] = {
+        { 100, "Continue" },
+        { 101, "Switching Protocols" },
+        { 200, "OK" },
+        { 201, "Created" },
+        { 202, "Accepted" },
+        { 203, "Non-Authoritative Information" },
+        { 204, "No Content" },
+        { 205, "Reset Content" },
+        { 206, "Partial Content" },
+        { 300, "Multiple Choices" },
+        { 301, "Moved Permanently" },
+        { 302, "Moved Temporarily" },
+        { 303, "See Other" },
+        { 304, "Not Modified" },
+        { 305, "Use Proxy" },
+        { 400, "Bad Request" },
+        { 401, "Unauthorized" },
+        { 402, "Payment Required" },
+        { 403, "Forbidden" },
+        { 404, "Not Found" },
+        { 405, "Method Not Allowed" },
+        { 406, "Not Acceptable" },
+        { 407, "Proxy Authentication Required" },
+        { 408, "Request Timeout" },
+        { 409, "Conflict" },
+        { 410, "Gone" },
+        { 411, "Length Required" },
+        { 412, "Precondition Failed" },
+        { 413, "Request Entity Too Large" },
+        { 414, "Request-URI Too Long" },
+        { 415, "Unsupported Media Type" },
+        { 500, "Internal Server Error" },
+        { 501, "Not Implemented" },
+        { 502, "Bad Gateway" },
+        { 503, "Service Unavailable" },
+        { 504, "Gateway Timeout" },
+        { 505, "HTTP Version Not Supported" },
+        { 000, NULL}
+    };
+
+    r = reasons;
+    while (r->status <= status)
+        if (r->status == status)
+            return r->reason;
+        else
+            r++;
+    return "No Reason";
+}
+
 static int escape_url(const char *path, char *dest, int destsize)
 {
     const BYTE *s = (const BYTE *)path;
@@ -361,21 +435,21 @@ static int escape_url(const char *path, char *dest, int destsize)
  */
 static char *stristr(const char *s, const char *find)
 {
-	char c, sc;
-	size_t len;
+    char c, sc;
+    size_t len;
 
-	if ((c = tolower((unsigned char)(*find++))) != 0) {
-		len = strlen(find);
-		do {
-			do {
-				if ((sc = tolower((unsigned char)(*s++))) == 0)
-					return (NULL);
-			} while (sc != c);
-		} while (strnicmp(s, find, len) != 0);
-		s--;
-	}
-	return ((char *)s);
-} 
+    if ((c = tolower((unsigned char)(*find++))) != 0) {
+        len = strlen(find);
+        do {
+            do {
+                if ((sc = tolower((unsigned char)(*s++))) == 0)
+                    return (NULL);
+            } while (sc != c);
+        } while (strnicmp(s, find, len) != 0);
+        s--;
+    }
+    return ((char *)s);
+}
 
 static int uri_is_web_inf(const char *uri)
 {
@@ -415,7 +489,7 @@ static int JK_METHOD start_response(jk_ws_service_t *s,
     JK_TRACE_ENTER(logger);
     if (status < 100 || status > 1000) {
         jk_log(logger, JK_LOG_ERROR,
-               "invalid status %d\n",
+               "invalid status %d",
                status);
         JK_TRACE_EXIT(logger);
         return JK_FALSE;
@@ -434,7 +508,7 @@ static int JK_METHOD start_response(jk_ws_service_t *s,
              * Create the status line
              */
             if (!reason) {
-                reason = "";
+                reason = status_reason(status);
             }
             status_str = (char *)_alloca((6 + strlen(reason)) * sizeof(char));
             sprintf(status_str, "%d %s", status, reason);
@@ -473,7 +547,7 @@ static int JK_METHOD start_response(jk_ws_service_t *s,
                                                  (LPDWORD) &len_of_status,
                                                  (LPDWORD) headers_str)) {
                 jk_log(logger, JK_LOG_ERROR,
-                       "HSE_REQ_SEND_RESPONSE_HEADER failed\n");
+                       "HSE_REQ_SEND_RESPONSE_HEADER failed");
                 JK_TRACE_EXIT(logger);
                 return JK_FALSE;
             }
@@ -508,7 +582,7 @@ static int JK_METHOD read(jk_ws_service_t *s,
             }
             else {
                 /*
-                 * Try to copy what we already have 
+                 * Try to copy what we already have
                  */
                 if (already_read > 0) {
                     memcpy(buf, p->lpEcb->lpbData + p->bytes_read_so_far,
@@ -528,7 +602,7 @@ static int JK_METHOD read(jk_ws_service_t *s,
                 }
                 else {
                     jk_log(logger, JK_LOG_ERROR,
-                           "ReadClient failed with %08x\n", GetLastError());
+                           "ReadClient failed with %08x", GetLastError());
                     JK_TRACE_EXIT(logger);
                     return JK_FALSE;
                 }
@@ -563,7 +637,7 @@ static int JK_METHOD write(jk_ws_service_t *s, const void *b, unsigned int l)
                 if (!p->lpEcb->WriteClient(p->lpEcb->ConnID,
                                            buf + written, &try_to_write, 0)) {
                     jk_log(logger, JK_LOG_ERROR,
-                           "WriteClient failed with %08x\n", GetLastError());
+                           "WriteClient failed with %08x", GetLastError());
                     JK_TRACE_EXIT(logger);
                     return JK_FALSE;
                 }
@@ -595,6 +669,7 @@ BOOL WINAPI GetFilterVersion(PHTTP_FILTER_VERSION pVer)
                     SF_NOTIFY_SECURE_PORT |
                     SF_NOTIFY_NONSECURE_PORT |
                     SF_NOTIFY_PREPROC_HEADERS |
+                    SF_NOTIFY_LOG |
                     SF_NOTIFY_AUTH_COMPLETE;
 
     strcpy(pVer->lpszFilterDesc, VERSION_STRING);
@@ -633,10 +708,10 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
             GetServerVariable(pfc, SERVER_SOFTWARE, serverSoftware, &dwLen)) {
             iis5 = (atof(serverSoftware + 14) >= 5.0);
             if (iis5) {
-                jk_log(logger, JK_LOG_DEBUG, "Detected IIS >= 5.0\n");
+                jk_log(logger, JK_LOG_DEBUG, "Detected IIS >= 5.0");
             }
             else {
-                jk_log(logger, JK_LOG_DEBUG, "Detected IIS < 5.0\n");
+                jk_log(logger, JK_LOG_DEBUG, "Detected IIS < 5.0");
             }
         }
     }
@@ -684,7 +759,7 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
         }
 
         if (JK_IS_DEBUG_LEVEL(logger))
-            jk_log(logger, JK_LOG_DEBUG, "Filter started\n");
+            jk_log(logger, JK_LOG_DEBUG, "Filter started");
 
         /*
          * Just in case somebody set these headers in the request!
@@ -696,13 +771,13 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
 
         if (!GetHeader(pfc, "url", (LPVOID) uri, (LPDWORD) & sz)) {
             jk_log(logger, JK_LOG_ERROR,
-                   "error while getting the url\n");
+                   "error while getting the url");
             return SF_STATUS_REQ_ERROR;
         }
 
         if (strlen(uri)) {
             int rc;
-            char *worker = 0;
+            const char *worker = NULL;
             query = strchr(uri, '?');
             if (query) {
                 *query++ = '\0';
@@ -711,7 +786,7 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
             rc = unescape_url(uri);
             if (rc == BAD_REQUEST) {
                 jk_log(logger, JK_LOG_ERROR,
-                       "[%s] contains one or more invalid escape sequences.\n",
+                       "[%s] contains one or more invalid escape sequences.",
                        uri);
                 write_error_response(pfc, "400 Bad Request",
                                      HTML_ERROR_400);
@@ -719,10 +794,10 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
             }
             else if (rc == BAD_PATH) {
                 jk_log(logger, JK_LOG_EMERG,
-                       "[%s] contains forbidden escape sequences.\n",
+                       "[%s] contains forbidden escape sequences.",
                        uri);
-                write_error_response(pfc, "403 Forbidden",
-                                     HTML_ERROR_403);
+                write_error_response(pfc, "404 Not Found",
+                                     HTML_ERROR_404);
                 return SF_STATUS_REQ_FINISHED;
             }
             getparents(uri);
@@ -751,14 +826,14 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                 strcat(snuri, uri);
                 if (JK_IS_DEBUG_LEVEL(logger))
                     jk_log(logger, JK_LOG_DEBUG,
-                           "Virtual Host redirection of %s\n",
+                           "Virtual Host redirection of %s",
                            snuri);
                 worker = map_uri_to_worker(uw_map, snuri, logger);
             }
             if (!worker) {
                 if (JK_IS_DEBUG_LEVEL(logger))
                     jk_log(logger, JK_LOG_DEBUG,
-                           "Default redirection of %s\n",
+                           "Default redirection of %s",
                            uri);
                 worker = map_uri_to_worker(uw_map, uri, logger);
             }
@@ -768,16 +843,16 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
              */
             if (JK_IS_DEBUG_LEVEL(logger))
                 jk_log(logger, JK_LOG_DEBUG,
-                       "check if [%s] is points to the web-inf directory\n",
+                       "check if [%s] is points to the web-inf directory",
                        uri);
 
             if (uri_is_web_inf(uri)) {
                 jk_log(logger, JK_LOG_EMERG,
-                       "[%s] points to the web-inf or meta-inf directory.\nSomebody try to hack into the site!!!\n",
+                       "[%s] points to the web-inf or meta-inf directory.\nSomebody try to hack into the site!!!",
                        uri);
 
-                write_error_response(pfc, "403 Forbidden",
-                                     HTML_ERROR_403);
+                write_error_response(pfc, "404 Not Found",
+                                     HTML_ERROR_404);
                 return SF_STATUS_REQ_FINISHED;
             }
 
@@ -786,7 +861,7 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
 
                 /* This is a servlet, should redirect ... */
                 jk_log(logger, JK_LOG_DEBUG,
-                       "[%s] is a servlet url - should redirect to %s\n",
+                       "[%s] is a servlet url - should redirect to %s",
                        uri, worker);
 
                 /* get URI we should forward */
@@ -798,14 +873,14 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                         *(query - 1) = '\0';
                     if (JK_IS_DEBUG_LEVEL(logger))
                         jk_log(logger, JK_LOG_DEBUG,
-                               "fowarding original URI [%s]\n",
+                               "fowarding original URI [%s]",
                                uri);
                     forwardURI = uri;
                 }
                 else if (uri_select_option == URI_SELECT_OPT_ESCAPED) {
                     if (!escape_url(uri, snuri, INTERNET_MAX_URL_LENGTH)) {
                         jk_log(logger, JK_LOG_ERROR,
-                               "[%s] re-encoding request exceeds maximum buffer size.\n",
+                               "[%s] re-encoding request exceeds maximum buffer size.",
                                uri);
                         write_error_response(pfc, "400 Bad Request",
                                              HTML_ERROR_400);
@@ -813,7 +888,7 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                     }
                     if (JK_IS_DEBUG_LEVEL(logger))
                         jk_log(logger, JK_LOG_DEBUG,
-                               "fowarding escaped URI [%s]\n",
+                               "fowarding escaped URI [%s]",
                                snuri);
                     forwardURI = snuri;
                 }
@@ -824,10 +899,10 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                 if (!AddHeader(pfc, URI_HEADER_NAME, forwardURI) ||
                     ((query != NULL && strlen(query) > 0)
                      ? !AddHeader(pfc, QUERY_HEADER_NAME, query) : FALSE) ||
-                    !AddHeader(pfc, WORKER_HEADER_NAME, worker) ||
+                    !AddHeader(pfc, WORKER_HEADER_NAME, (LPSTR)worker) ||
                     !SetHeader(pfc, "url", extension_uri)) {
                     jk_log(logger, JK_LOG_ERROR,
-                           "error while adding request headers\n");
+                           "error while adding request headers");
                     return SF_STATUS_REQ_ERROR;
                 }
 
@@ -836,25 +911,47 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                  * This allows the servlet to handle 'Translate: f'.
                  */
                 if (GetHeader
-                    (pfc, "Translate:", (LPVOID) Translate,
+                    (pfc, TRANSLATE_HEADER, (LPVOID) Translate,
                      (LPDWORD) & szTranslate) && Translate != NULL
                     && szTranslate > 0) {
                     if (!AddHeader
                         (pfc, TOMCAT_TRANSLATE_HEADER_NAME, Translate)) {
                         jk_log(logger, JK_LOG_ERROR,
-                               "error while adding Tomcat-Translate headers\n");
+                               "error while adding Tomcat-Translate headers");
                         return SF_STATUS_REQ_ERROR;
                     }
                     SetHeader(pfc, "Translate:", NULL);
                 }
+                if (!pfc->pFilterContext) {
+                    isapi_log_data_t *ld = (isapi_log_data_t *)pfc->AllocMem(pfc, sizeof(isapi_log_data_t), 0);
+                    if (!ld) {
+                        jk_log(logger, JK_LOG_ERROR,
+                               "error while allocating memory");
+                        SetLastError(ERROR_NOT_ENOUGH_MEMORY);
+                        return SF_STATUS_REQ_ERROR;
+                    }
+                    memset(ld, 0, sizeof(isapi_log_data_t));
+                    strcpy(ld->uri, forwardURI);
+                    if (query && strlen(query) > 0)
+                        strcpy(ld->query, query);
+                    pfc->pFilterContext = ld;
+                }
             }
             else {
                 if (JK_IS_DEBUG_LEVEL(logger))
                     jk_log(logger, JK_LOG_DEBUG,
-                           "[%s] is not a servlet url\n", uri);
+                           "[%s] is not a servlet url", uri);
             }
         }
     }
+    else if (is_inited && (dwNotificationType == SF_NOTIFY_LOG)) {
+        if (pfc->pFilterContext) {
+            isapi_log_data_t *ld = (isapi_log_data_t *)pfc->pFilterContext;
+            HTTP_FILTER_LOG  *pl = (HTTP_FILTER_LOG *)pvNotification;
+            pl->pszTarget = ld->uri;
+            pl->pszParameters = ld->query;
+        }
+    }
     return SF_STATUS_REQ_NEXT_NOTIFICATION;
 }
 
@@ -903,6 +1000,7 @@ DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpEcb)
         jk_pool_atom_t buf[SMALL_POOL_SIZE];
         char *worker_name;
 
+        wc_maintain(logger);
         jk_init_ws_service(&s);
         jk_open_pool(&private_data.p, buf, sizeof(buf));
 
@@ -918,31 +1016,32 @@ DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpEcb)
 
             if (JK_IS_DEBUG_LEVEL(logger))
                 jk_log(logger, JK_LOG_DEBUG,
-                       "%s a worker for name %s\n",
+                       "%s a worker for name %s",
                        worker ? "got" : "could not get", worker_name);
 
             if (worker) {
                 jk_endpoint_t *e = NULL;
                 /* Update retries for this worker */
-                s.retries = worker->retries;                
+                s.retries = worker->retries;
                 if (worker->get_endpoint(worker, &e, logger)) {
-                    int recover = JK_FALSE;
-                    if (e->service(e, &s, logger, &recover)) {
+                    int is_error = JK_HTTP_SERVER_ERROR;
+                    if (e->service(e, &s, logger, &is_error)) {
                         rc = HSE_STATUS_SUCCESS;
                         lpEcb->dwHttpStatusCode = HTTP_STATUS_OK;
                         jk_log(logger, JK_LOG_DEBUG,
-                               "service() returned OK\n");
+                               "service() returned OK");
                     }
                     else {
+                        lpEcb->dwHttpStatusCode = is_error;
                         jk_log(logger, JK_LOG_ERROR,
-                               "service() failed\n");
+                               "service() failed");
                     }
                     e->done(&e, logger);
                 }
             }
             else {
                 jk_log(logger, JK_LOG_ERROR,
-                       "could not get a worker for name %s\n",
+                       "could not get a worker for name %s",
                        worker_name);
             }
         }
@@ -950,7 +1049,7 @@ DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpEcb)
     }
     else {
         jk_log(logger, JK_LOG_ERROR,
-               "not initialized\n");
+               "not initialized");
     }
 
     JK_TRACE_EXIT(logger);
@@ -975,6 +1074,10 @@ BOOL WINAPI TerminateFilter(DWORD dwFlags)
             uri_worker_map_free(&uw_map, logger);
             is_mapread = JK_FALSE;
         }
+        if (workers_map) {
+            jk_map_free(&workers_map);
+            workers_map = NULL;
+        }
         wc_close(logger);
         if (logger) {
             jk_close_file_logger(&logger);
@@ -1006,6 +1109,16 @@ BOOL WINAPI DllMain(HINSTANCE hInst,    // Instance Handle of the DLL
         else {
             fReturn = JK_FALSE;
         }
+        /* Construct redirector headers to use for this redirector instance */
+        sprintf(URI_HEADER_NAME, HEADER_TEMPLATE, URI_HEADER_NAME_BASE, hInst);
+        sprintf(QUERY_HEADER_NAME, HEADER_TEMPLATE, QUERY_HEADER_NAME_BASE, hInst);
+        sprintf(WORKER_HEADER_NAME, HEADER_TEMPLATE, WORKER_HEADER_NAME_BASE, hInst);
+        sprintf(TOMCAT_TRANSLATE_HEADER_NAME, HEADER_TEMPLATE, TOMCAT_TRANSLATE_HEADER_NAME_BASE, hInst);
+
+        sprintf(HTTP_URI_HEADER_NAME, HTTP_HEADER_TEMPLATE, URI_HEADER_NAME_BASE, hInst);
+        sprintf(HTTP_QUERY_HEADER_NAME, HTTP_HEADER_TEMPLATE, QUERY_HEADER_NAME_BASE, hInst);
+        sprintf(HTTP_WORKER_HEADER_NAME, HTTP_HEADER_TEMPLATE, WORKER_HEADER_NAME_BASE, hInst);
+
     break;
     case DLL_PROCESS_DETACH:
         __try {
@@ -1025,84 +1138,63 @@ BOOL WINAPI DllMain(HINSTANCE hInst,    // Instance Handle of the DLL
 static int init_jk(char *serverName)
 {
     int rc = JK_FALSE;
-    jk_map_t *map;
 
     if (!jk_open_file_logger(&logger, log_file, log_level)) {
         logger = NULL;
     }
+     /* Simulate shared memory
+      * For now use fixed size.
+      */
+     jk_shm_open(SHM_DEF_NAME, JK_SHM_DEF_SIZE, logger);
+
+     /* 10 is minimum supported on WINXP */
+     jk_set_worker_def_cache_size(10);
+
     /* Logging the initialization type: registry or properties file in virtual dir
      */
     if (JK_IS_DEBUG_LEVEL(logger)) {
         if (using_ini_file) {
-            jk_log(logger, JK_LOG_DEBUG, "Using ini file %s.\n", ini_file_name);
+            jk_log(logger, JK_LOG_DEBUG, "Using ini file %s.", ini_file_name);
         }
         else {
-            jk_log(logger, JK_LOG_DEBUG, "Using registry.\n");
+            jk_log(logger, JK_LOG_DEBUG, "Using registry.");
         }
 
-        jk_log(logger, JK_LOG_DEBUG, "Using log file %s.\n", log_file);
-        jk_log(logger, JK_LOG_DEBUG, "Using log level %d.\n", log_level);
-        jk_log(logger, JK_LOG_DEBUG, "Using extension uri %s.\n", extension_uri);
-        jk_log(logger, JK_LOG_DEBUG, "Using worker file %s.\n", worker_file);
-        jk_log(logger, JK_LOG_DEBUG, "Using worker mount file %s.\n",
+        jk_log(logger, JK_LOG_DEBUG, "Using log file %s.", log_file);
+        jk_log(logger, JK_LOG_DEBUG, "Using log level %d.", log_level);
+        jk_log(logger, JK_LOG_DEBUG, "Using extension uri %s.", extension_uri);
+        jk_log(logger, JK_LOG_DEBUG, "Using worker file %s.", worker_file);
+        jk_log(logger, JK_LOG_DEBUG, "Using worker mount file %s.",
                worker_mount_file);
-        jk_log(logger, JK_LOG_DEBUG, "Using uri select %d.\n", uri_select_option);
+        jk_log(logger, JK_LOG_DEBUG, "Using uri select %d.", uri_select_option);
     }
-    if (jk_map_alloc(&map)) {
-        if (jk_map_read_properties(map, worker_mount_file)) {
-            /* remove non-mapping entries (assume they were string substitutions) */
-            jk_map_t *map2;
-            if (jk_map_alloc(&map2)) {
-                int sz, i;
-                void *old;
-
-                sz = jk_map_size(map);
-                for (i = 0; i < sz; i++) {
-                    char *name = jk_map_name_at(map, i);
-                    if (*name == '/' || *name == '!') {
-                        jk_map_put(map2, name, jk_map_value_at(map, i), &old);
-                    }
-                    else {
-                        if (JK_IS_DEBUG_LEVEL(logger))
-                            jk_log(logger, JK_LOG_DEBUG,
-                                   "Ignoring worker mount file entry %s=%s.\n",
-                                   name, jk_map_value_at(map, i));
-                    }
-                }
-
-                if (uri_worker_map_alloc(&uw_map, map2, logger)) {
-                    rc = JK_TRUE;
-                }
-
-                jk_map_free(&map2);
-            }
-        }
-        else {
-            jk_log(logger, JK_LOG_EMERG,
-                   "Unable to read worker mount file %s.\n",
-                   worker_mount_file);
-        }
-        jk_map_free(&map);
+    if (uri_worker_map_alloc(&uw_map, NULL, logger)) {
+        rc = JK_FALSE;
+        uw_map->fname = worker_mount_file;
+        if (worker_mount_file[0])
+            rc = uri_worker_map_load(uw_map, logger);
     }
-
     if (rc) {
         rc = JK_FALSE;
-        if (jk_map_alloc(&map)) {
-            if (jk_map_read_properties(map, worker_file)) {
+        if (jk_map_alloc(&workers_map)) {
+            if (jk_map_read_properties(workers_map, worker_file, NULL)) {
                 /* we add the URI->WORKER MAP since workers using AJP14 will feed it */
 
                 worker_env.uri_to_worker = uw_map;
                 worker_env.server_name = serverName;
 
-                if (wc_open(map, &worker_env, logger)) {
+                if (wc_open(workers_map, &worker_env, logger)) {
                     rc = JK_TRUE;
                 }
             }
             else {
                 jk_log(logger, JK_LOG_EMERG,
-                       "Unable to read worker file %s.\n", worker_file);
+                       "Unable to read worker file %s.", worker_file);
+            }
+            if (rc != JK_TRUE) {
+                jk_map_free(&workers_map);
+                workers_map = NULL;
             }
-            jk_map_free(&map);
         }
     }
 
@@ -1141,11 +1233,11 @@ static int read_registry_init_data(void)
     HKEY hkey;
     long rc;
     int ok = JK_TRUE;
-    char *tmp;
+    const char *tmp;
     jk_map_t *map;
 
     if (jk_map_alloc(&map)) {
-        if (jk_map_read_properties(map, ini_file_name)) {
+        if (jk_map_read_properties(map, ini_file_name, NULL)) {
             using_ini_file = JK_TRUE;
         }
     }
@@ -1295,6 +1387,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
     s->start_response = start_response;
     s->read = read;
     s->write = write;
+    s->flush = NULL;
 
     /* Clear RECO status */
     s->reco_status = RECO_NONE;
@@ -1334,7 +1427,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
     s->headers_names = NULL;
     s->headers_values = NULL;
     s->num_headers = 0;
-
+    s->uw_map = uw_map;
     /*
      * Add SSL IIS environment
      */
@@ -1399,7 +1492,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
                                           (LPVOID) & cc, NULL,
                                           NULL) != FALSE) {
                     jk_log(logger, JK_LOG_DEBUG,
-                           "Client Certificate encoding:%d sz:%d flags:%ld\n",
+                           "Client Certificate encoding:%d sz:%d flags:%ld",
                            cc.CertContext.
                            dwCertEncodingType & X509_ASN_ENCODING,
                            cc.CertContext.cbCertEncoded,
@@ -1420,7 +1513,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
 
     huge_buf_sz = sizeof(huge_buf);
     if (get_server_value(private_data->lpEcb,
-                         "ALL_HTTP", huge_buf, huge_buf_sz, "")) {
+                         "ALL_HTTP", huge_buf, huge_buf_sz)) {
         unsigned int cnt = 0;
         char *tmp;
 
@@ -1436,7 +1529,10 @@ static int init_ws_service(isapi_private_data_t * private_data,
             size_t len_of_http_prefix = strlen("HTTP_");
             BOOL need_content_length_header = (s->content_length == 0);
 
-            cnt -= 2;           /* For our two special headers */
+            cnt -= 2;           /* For our two special headers:
+                                 * HTTP_TOMCATURI_XXXXXXXX
+                                 * HTTP_TOMCATWORKER_XXXXXXXX
+                                 */
             /* allocate an extra header slot in case we need to add a content-length header */
             s->headers_names =
                 jk_pool_alloc(&private_data->p, (cnt + 1) * sizeof(char *));
@@ -1458,6 +1554,14 @@ static int init_ws_service(isapi_private_data_t * private_data,
                                  strlen(WORKER_HEADER_NAME))) {
                     real_header = JK_FALSE;
                 }
+                else if (!strnicmp(tmp, QUERY_HEADER_NAME,
+                                   strlen(QUERY_HEADER_NAME))) {
+                    /* HTTP_TOMCATQUERY_XXXXXXXX was supplied,
+                     * remove it from the count and skip
+                     */
+                    cnt--;
+                    real_header = JK_FALSE;
+                }
                 else if (need_content_length_header &&
                          !strnicmp(tmp, CONTENT_LENGTH,
                                    strlen(CONTENT_LENGTH))) {
@@ -1466,8 +1570,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
                 }
                 else if (!strnicmp(tmp, TOMCAT_TRANSLATE_HEADER_NAME,
                                    strlen(TOMCAT_TRANSLATE_HEADER_NAME))) {
-                    tmp += 6;   /* TOMCAT */
-                    s->headers_names[i] = tmp;
+                    s->headers_names[i] = TRANSLATE_HEADER_NAME_LC;
                 }
                 else {
                     s->headers_names[i] = tmp;
@@ -1533,18 +1636,16 @@ static int init_ws_service(isapi_private_data_t * private_data,
 }
 
 static int get_server_value(LPEXTENSION_CONTROL_BLOCK lpEcb,
-                            char *name, char *buf, DWORD bufsz, char *def_val)
+                            char *name, char *buf, DWORD bufsz)
 {
-    if (!lpEcb->GetServerVariable(lpEcb->ConnID,
-                                  name, buf, (LPDWORD) &bufsz)) {
-        strcpy(buf, def_val);
+    DWORD sz = bufsz;
+    buf[0]   = '\0';
+    if (!lpEcb->GetServerVariable(lpEcb->ConnID, name,
+                                  buf, (LPDWORD) &sz))
         return JK_FALSE;
-    }
-
-    if (bufsz > 0) {
-        buf[bufsz - 1] = '\0';
-    }
 
+    if (sz <= bufsz)
+        buf[sz-1] = '\0';
     return JK_TRUE;
 }
 
